/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.ContextSwitcher");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.util.html");
dojo.requireLocalization("jazz.app.internal", "ContextSwitcherMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var messages = dojo.i18n.getLocalization("jazz.app.internal", "ContextSwitcherMessages");

dojo.declare("jazz.app.internal.ContextSwitcher", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("jazz.app", "internal/templates/ContextSwitcher.html"),
	
	/**
	 * Fill in the dynamic elements of the header nav.
	 */
	postCreate: function() {
		if (this.currentContext){
			if (this.currentContext.name) {
				this._addCurrentContext();
			} else {
				this.currentContext = null;
				this._addSelectContext();
			}
		} else {
			this._addSelectContext();
		}
		if (!this.contexts) {
			dojo.addClass(this.domNode,"jazz-app-projectSelector-grayed");
			if (this.contextProvider) {
				this._addLoading();
				this._onclickHandle = this.connect(this.domNode, "onclick", this._callContextProvider);
				this._timeoutHandle = setTimeout(dojo.hitch(this, this._callContextProvider), 6000);	
			} else {
				this._noContexts();
			}
		} else if (this.contexts.length > 0) {
			if (this.currentContext) {
				var selectIndex = 1;	
			} else {
				var selectIndex = 2;
				dojo.addClass(this.domNode,"jazz-app-projectSelector-grayed");
			}
			for (var i = 0; i < this.contexts.length; i++) {
				if (!this.contexts[i].archived && (!this.currentContext || this.currentContext.name != this.contexts[i].name)) {
					this._addContext(this.contexts[i],selectIndex);
					selectIndex++;
				}
			}
		}
		this._addAdminOption();
		this.connect(this.domNode, "onchange", this._onChange);	
		this.selectedIndex = this.domNode.selectedIndex;
		dojo.subscribe(jazz.app.events.contextSwitcher.deleteContext, this, "deleteContext");
		dojo.subscribe(jazz.app.events.contextSwitcher.addContext, this, "addContext");
	},
	populateContexts: function(contexts){
		this._populateContexts(contexts);
	},
	addContext: function(contextName) {
		this.contexts.push({name: contextName});
		this._populateContexts(this.contexts);
	},
	deleteContext: function(contextName) {
		var success = false;
		for(var i = 0; i < this.contexts.length; i++) {
			if (this.contexts[i].name === contextName) {
				this.contexts.splice(i,1);
				success = true;
				break;
			}
		}
		if (success) {
			this._populateContexts(this.contexts);	
		}
		return success;
	},
	_callContextProvider: function() {
		try {
			if (this._contextsFetched) return;
			clearTimeout(this._timeoutHandle);
			this.disconnect(this._onclickHandle);
			this._timeoutHandle = null;
			this._onclickHandle = null;
			var dfd = new dojo.Deferred();
			dfd.addCallback(dojo.hitch(this,this._populateContexts));
			dfd.addErrback(dojo.hitch(this,this._noContexts));
			this.contextProvider(dfd);
			this._contextsFetched = true;
		} catch(e) {
			console.error(e);
			//eat the error, add empty project switcher
			this._noContexts();
		}
	},
	_populateContexts: function(contexts) {
		this.contexts = contexts;
		if(contexts == null || contexts.length === 0){
			this._noContexts();
			this._addAdminOption();
			return;
		}
		if (contexts.length === 1) {
			this.domNode.options[1] = null;
		}
		dojo.removeClass(this.domNode,"jazz-app-projectSelector-grayed");
		var contextsToAdd = new Array();
		for (var i = 0; i < contexts.length; i++) {
			if (!contexts[i].archived) {
				contextsToAdd.push(contexts[i]);
			}
		}
		if (contextsToAdd.length == 0) {
			this._noContexts(true);
			return;	
		}
		var options = this.domNode.options;

		if (this.adminUI || this.admin) {
			options[options.length-1] = null;
			options[options.length-1] = null;
		} 
		if (options.length > 0) {
			options[options.length-1] = null;
		}
	
		var selectIndex = 0;
		if (!this.currentContext && !this.adminUI) {
			this._addSelectContext();
		}
		
		for (var i = 0; i < contextsToAdd.length; i++) {
			this._addContext(contextsToAdd[i],selectIndex);
			selectIndex++;
		}
		this._addAdminOption();
		this.selectedIndex = this.domNode.selectedIndex;
	},
	_addSelectContext: function() {
		if (this.selectLabel) {
			var label = jazz.util.html.escape(this.selectLabel);
		} else {
			var label = "";	
		}
		var option = new Option(label, label, true, true);
		option.className = "separator";
		this.domNode.options[0] = option;
		this._addSeparator();
		selectIndex = 2;
	},
	_addContext: function(context, i) {
		if (this.currentContext && (context.name.toLowerCase() == this.currentContext.name.toLowerCase() || "the " + context.name.toLowerCase() == this.currentContext.name.toLowerCase())) {
			this.domNode.options[i] = new Option(context.name,context.name,true,true);
			this.domNode.options[i].className = "selected";
			this.domNode.selectedIndex = i;
		} else 
			this.domNode.options[i] = new Option(context.name,context.name);
	},
	_onChange: function(event) {
		var context;
		var contextName = this.domNode.options[this.domNode.selectedIndex].value;
		if (contextName == "disabled") {
			this.domNode.options[this.selectedIndex].selected = "selected";
			return;
		}
		if (this.contexts) {
			for (var i = 0; i < this.contexts.length; i++) {
				if (contextName === this.contexts[i].name) {
					context = this.contexts[i];
				}
			}
		}
		if (!context) {
			context = {name: contextName};
			if (contextName == messages.admin) {
				context.admin = true;
			}
		}		

		if (this.onChange) {
			try {
				var href = this.onChange(context);
			} catch(e) {
				this._selectEntry(this.selectedIndex);
				console.error(e);
				return;	
			}
		}
		if (href && PlatformUI.getWorkbench().shouldVetoNavigation(href)) {
			this._selectEntry(this.selectedIndex);
		} else {
			if (contextName == messages.projectAreas) {
				this._selectEntry(this.selectedIndex); //re-highlight current project area
			} else {
				dojo.removeClass(this.domNode,"jazz-app-projectSelector-grayed");
				// highlight selected entry and store index
				this.domNode.options[this.selectedIndex].className = "";
				this.selectedIndex = this.domNode.selectedIndex;
				this.domNode.options[this.selectedIndex].className = "selected";
			}
			if (href) {
				window.location.href = href;	
			}
		}
	},
	_selectEntry: function(index) {
		this.domNode.options[this.domNode.selectedIndex].className = "";
		this.domNode.options[index].className = "selected";
		this.domNode.options[index].selected = "selected";
	},
	_addLoading: function() {
		var loading = new Option(messages.loading, "disabled");
		loading.className = "separator";
		this.domNode.options[this.domNode.options.length] = loading;
	},
	_addSeparator: function() {
		var separator = new Option("-------------------------------","disabled");
		separator.disabled = "disabled";
		separator.className = "separator";
		this.domNode.options[this.domNode.options.length] = separator;
	},
	_addCurrentContext: function() {
		if (!this.adminUI && this.currentContext) {
			var currentContext = this.currentContext;
			if (currentContext.archived) {
				var option = new Option(dojo.string.substitute(messages.archivedProject, [ currentContext.name ]), dojo.string.substitute(messages.archivedProject, [ currentContext.name ]), true, true);	
			} else {
				var option = new Option(currentContext.name, currentContext.name, true, true);
			}
			option.className = "selected";
			this.domNode.options[this.domNode.options.length] = option;
			this.domNode.selectedIndex = this.domNode.options.length - 1;
		}
	},
	_addAdminOption: function() {
		if (this.admin || this.adminUI) {
			this._addSeparator();
			if (this.adminUI) {
				var option = new Option(messages.admin, messages.admin, true, true);
				option.className = "selected";
				this.domNode.options[this.domNode.options.length] = option;
				this.selectedIndex = this.domNode.selectedIndex = this.domNode.options.length - 1;
				dojo.removeClass(this.domNode,"jazz-app-projectSelector-grayed");
			} else 
				this.domNode.options[this.domNode.options.length] = new Option(messages.admin, messages.admin);
		}
	},
	_noContexts: function () {
		if (this.emptyLabel) {
			var label = jazz.util.html.escape(this.emptyLabel); 
		} else {
			var label = "";
		}
		this.domNode.options[0].innerHTML = label;
		this.domNode.options[0].text = label;
		this.domNode.options[0].disabled = "disabled";
		if(!this.adminUI) {
			dojo.addClass(this.domNode.options[0],"selected");	
		}
		if (this.domNode.selectedIndex == 0) {
			dojo.addClass(this.domNode,"jazz-app-projectSelector-grayed");
		}
		this.domNode.options[1] = null;
		this.domNode.options[1] = null;
	}
});

var EVENTS_ROOT = "jazz.app.events.contextSwitcher";
dojo.setObject(EVENTS_ROOT, {deleteContext: EVENTS_ROOT + ".deleteContext", addContext: EVENTS_ROOT + ".addContext"});

})();
