/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.ProxyRequestInterceptor");

var encodeProxiedURIFunction = null;

/**
 * Checks to see if the incoming request contains a URL with a different origin
 * than the current page (that is either a different scheme, host, or port). If
 * the request URL has a different origin, this request will rewrite the URL to
 * go to the Jazz Web UI Server proxy.
 * 
 * @param {Object} request the request to check for different origin
 */
jazz.app.internal.ProxyRequestInterceptor = function(request){
	var url = dojo.getObject("args.url", false, request);
	if(!url) {
		return; // no url argument, early return
	}
	var urlObj = new dojo._Url(request.args.url);
	if(isDifferentOrigin(urlObj)) {
		request.args.url = encodeProxiedURIFunction(request.args.url);
		//console.warn("New proxy URL: " + request.args.url);
	}
};

/**
 * Sets the function that will be used to construct proxy URIs.
 * 
 * @param {Function} paramencodeProxiedURIFunction the function that will be used to
 *  construct proxy URIs.
 */
jazz.app.internal.ProxyRequestInterceptor.setEncodeProxiedURIFunction = function(paramEncodeProxiedURIFunction) {
	encodeProxiedURIFunction = paramEncodeProxiedURIFunction;
};

/**
 * Checks the URL object's port property and if the port property is not set,
 * sets it to either "80" (HTTP) or "443" (HTTPS).
 * 
 * @param {Object} url the URL object to check and perhaps normalize
 */
function normalizePort(url) {
	var scheme = url.scheme || "http";
	if(!url.port) {
		url.port = scheme === "http" ? "80" : "443";
	}
}

// normalize the port property of the current page for later comparison with
// incoming URLs
var LOCAL_URL = new dojo._Url(location.href); 
normalizePort(LOCAL_URL);

/**
 * Returns true if the URL object is from a different origin than the current
 * page, otherwise false.
 * 
 * @param {Object} url the URL object to check
 */
function isDifferentOrigin(url) {
	
	// optimize for the most common case (request to a non-complete URL)
	if(!url.host) {
		// if host isn't set, scheme and port don't even make sense
		return false;
	}
	
	normalizePort(url);
	
	// if any of these are different, it's different origin
	var props = ["scheme", "host", "port"];
	for(var i=0; i<props.length; i++) {
		var prop = props[i];
		if(url[prop] !== LOCAL_URL[prop]) {
			return true;
		}
	}
	
	// all properties are equal, same origin
	return false;
}

})();