/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.wizard.Wizard");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.styledBox.TrimStyles");
dojo.require("jazz.ui.StyledBox");
dojo.require("com.ibm.team.repository.web.ui.wizard.UIHelper")

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojox.data.dom");

dojo.requireLocalization("com.ibm.team.repository.web.ui.wizard", "WizardMessages");

(function() {
	
var UIHelper = com.ibm.team.repository.web.ui.wizard.UIHelper;
var TrimStyles = jazz.ui.styledBox.TrimStyles;
var DEFAULT_LENGTH = 48;
var PADDING = 20;

dojo.declare("com.ibm.team.repository.web.ui.wizard.Wizard", null, {
	
	fullPageContainer: null,
	onFinish: null,
	onCancel: null,
	width: "944px",
	
	// passed on to wizard content
	contentArgs: {
		steps: [],
		showStepSummary: true,
		footerNode: null,
		headerNode: null,
		customProcessFinish: null, 
		customProcessCancel: null
	},

	constructor: function(args) {
		dojo.mixin(this.contentArgs, args);
		this.contentArgs.container = this;
		this.contentWidget = new com.ibm.team.repository.web.ui.wizard.WizardContent(this.contentArgs);
		var boxArgs = {
			contentNode: this.contentWidget.domNode,
			closable: false,
			closeOnEsc: false
		};
		if (args.fullPageContainer && isNode(args.fullPageContainer)) {
			this.containerNode = args.fullPageContainer;
			dojo.toggleClass(this.contentWidget.wizardCancelButton, "noDisplay", true);
			boxArgs.trim = TrimStyles.WIZARD;
			this.viewBox = new jazz.ui.StyledBox(boxArgs);
			this.containerNode.appendChild(this.viewBox.domNode);
			var includeCancel = false;
			this.contentWidget.resizeButtons(includeCancel);
		} else {
			var self = this;
			boxArgs.onOpen = function() {
				var includeCancel = true;
				self.contentWidget.resizeButtons(includeCancel);
			};
			boxArgs.primaryTitle = args.primaryTitle || "";
			boxArgs.width= this.width;
			dojo.toggleClass(this.contentWidget.wizardHRline, "noDisplay", true);
			this.viewBox = new jazz.ui.Dialog(boxArgs);
		}
	},
	
	addStep: function(step) {
		this.contentWidget.addStep(step);
	}, 
	
	selectStep: function(position) {
		this.contentWidget.selectStep(position);
	}, 
	
	close: function() {
		this.viewBox.close();
	}
	
});

dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepList");

dojo.declare("com.ibm.team.repository.web.ui.wizard.WizardContent", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/wizard/templates/Wizard.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.wizard", "WizardMessages");
		this.selectedStepPosition = 0;
		this._steps = [];
	},
	
	postCreate: function(){
		if (this.headerNode  && isNode(this.headerNode)) {
			this.wizardHeaderArea.appendChild(this.footerNode);
		}
		if (this.footerNode  && isNode(this.footerNode)) { 
			this.wizardFooterArea.appendChild(this.footerNode);
		}
		
		this._stepList = new com.ibm.team.repository.web.ui.wizard.WizardStepList();
		this.wizardStepArea.appendChild(this._stepList.domNode);
		
		if (!this.showStepSummary){
			this.hideStepList();
		}
		
		if(this.steps && this.steps.length > 0) {
			var self = this;
			dojo.forEach(this.steps, function(step){
				self.addStep(step);
			});
			this.selectStep(1);
		};
	},
	
	showStepList: function() {
		this._stepList.domNode.style.display = "block";
	},
	
	hideStepList: function() {
		this._stepList.domNode.style.display = "none";
	},
	
	getSelectedStep: function() {
		return this._steps[this.selectedStepPosition - 1];
	},
	
	addStep: function(step) {
	
		this._steps.push(step);
		step.position = this._steps.length;
		step.wizard = this;
		if (step.inStepList) {
			this._stepList.addStep(step);
		}	
		if (this._steps.length > 1) {
			dojo.toggleClass(this.wizardPreviousButton, "noDisplay", false);
			dojo.toggleClass(this.wizardNextButton, "noDisplay", false);
		}
	},
	
	selectStep: function(stepPosition) {
		if (stepPosition > this._steps.length || stepPosition < 1){
			// show error
			return;
		}
		this.selectedStepPosition = stepPosition;
		this.getSelectedStep().focusStep();
		this.updateSummaryHighlight();
		this.updateNavigationControls();
		this.updateMessages();
		this.displayStepContent();		
	},
	
	trySelectStep: function(stepPosition) {
		if (stepPosition > this._steps.length || stepPosition < 1){
			return false;
		}
		if (this._steps[stepPosition - 1].enabled) {
			this.selectStep(stepPosition);
			return true;
		}
		return false;
	},
	
	updateSummaryHighlight: function(){
		this._stepList.selectStep(this.getSelectedStep());
	},
	
	updateNavigationControls: function(){
		var step = this.getSelectedStep();
		this.wizardPreviousButton.disabled = !(step.canFlipToPreviousPage() && this.selectedStepPosition > 1);
		this.wizardNextButton.disabled = !(step.canFlipToNextPage() && this._steps.length > this.selectedStepPosition);
		this.wizardFinishButton.disabled = !step.canFinish();
		this.wizardCancelButton.disabled = !step.canCancel();
		dojo.toggleClass(this.wizardButtonArea, "noDisplay", !step.includeNavigationUI);
	},
	
	updateMessages: function(){},
	
	displayStepContent: function(){
		dojox.data.dom.removeChildren(this.wizardPanelArea);
		this.wizardPanelArea.appendChild(this.getSelectedStep().domNode);
	},
				
	onNavigationButtonClick: function(e) {
		var button = e.target;
		var step = this.getSelectedStep();
		if (button === this.wizardPreviousButton && step.canFlipToPreviousPage()) {
			step.onPrevious();
		} else if (button === this.wizardNextButton && step.canFlipToNextPage()){
			step.onNext();
		} else if (button === this.wizardFinishButton && step.canFinish()) {
			step.onFinish();
		} else if (button === this.wizardCancelButton && step.canCancel()) {
			step.onCancel();
		}	
	},
	
	processPrevious: function() {
		this.selectStep(this.selectedStepPosition - 1);
	}, 
	
	processNext: function() {
		this.selectStep(this.selectedStepPosition + 1);
	}, 
	
	processFinish: function() {
		if (this.customProcessFinish && dojo.isFunction(this.customProcessFinish)) {
         	this.customProcessFinish();
         }
		 this.container.close();
	}, 
	
	processCancel: function() {
		if (this.customProcessCancel && dojo.isFunction(this.customProcessCancel)) {
         	this.customProcessCancel();
      	}
		this.container.close();
	},	

	resizeButtons: function(includeCancel) {
		var tester = document.createElement("div"); 
		document.body.appendChild(tester);
		
		var words = [messages.previous,messages.next, messages.finish];
		if (includeCancel) {
			words.push(messages.cancel);
		}
		var newWidth = UIHelper.findMaxStringDisplayWidth(words, tester);
		
		if (newWidth > DEFAULT_LENGTH) {
			newWidth += PADDING;
			var newWidthStyle = newWidth + "pt";
			this.wizardPreviousButton.style.width = newWidthStyle;
			this.wizardNextButton.style.width = newWidthStyle;
			this.wizardFinishButton.style.width = newWidthStyle;
			if (includeCancel) {
				this.wizardCancelButton.style.width = newWidthStyle;
			}
		}
	}
	
});

function isNode(o){
	return o && o.nodeType && o.cloneNode;
}

})();